from django.shortcuts import render
from django.views.generic import View
from django.utils.decorators import method_decorator
from django.views.decorators.clickjacking import xframe_options_exempt

class SilentCheckSSOView(View):
    @method_decorator(xframe_options_exempt)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)
        
    def get(self, request, *args, **kwargs):
        return render(request, "authentication/silent-check-sso.html", locals())

class LoginView(View):
    def get(self, request, *args, **kwargs):
        return render(request, "authentication/login.html", locals())


from django.contrib import admin
from django.urls import path, include
from authentication import views

urlpatterns = [
    path('silent-check-sso/', views.SilentCheckSSOView.as_view(), name="silent-check-sso"),
    path('login/', views.LoginView.as_view(), name="login"),
]


from mozilla_django_oidc.auth import OIDCAuthenticationBackend
import logging

logger = logging.getLogger(__name__)

class ExtraClaimsOIDCAuthenticationBackend(OIDCAuthenticationBackend):
    def create_user(self, claims):
        """Return object for a newly created user account."""
        
        email = claims.get('email')
        first_name = claims.get("given_name", None)
        last_name = claims.get("family_name", None)
        is_staff = claims.get("is_staff", False)

        username = self.get_username(claims)

        return self.UserModel.objects.create_user(username, email, first_name=first_name, last_name=last_name, is_staff=is_staff)
    
    def update_user(self, user, claims):
        has_changes = False

        email = claims.get('email')
        first_name = claims.get("given_name", None)
        last_name = claims.get("family_name", None)
        is_staff = claims.get("is_staff", False)
        
        if user.email != email:
            logger.info("Updating user email...")
            user.email = email
            has_changes = True
        
        if user.first_name != first_name:
            logger.info("Updating user first_name...")
            user.first_name = first_name
            has_changes = True
        
        if user.last_name != last_name:
            logger.info("Updating user last_name...")
            user.last_name = last_name
            has_changes = True
        
        if user.is_staff != is_staff:
            logger.info("Updating user staff status...")
            user.is_staff = is_staff
            has_changes = True
        
        if has_changes:
            user.save()

        return user


import requests
import logging
from django.conf import settings
from django.contrib.auth.models import AbstractUser
from django.db import models

logger = logging.getLogger(__name__)

# Create your models here.
class User(AbstractUser):

    @staticmethod
    def get_context(email):
        context = {}
        try:
            response = requests.get(settings.USER_CONTEXT_URL, params={ "email" : email }, timeout=settings.USER_CONTEXT_TIMEOUT)
            response.raise_for_status()
            return response.json()
        except Exception:
            logger.exception("Occured an error getting the context of the user")
        
        return context

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.context = {}

    def __str__(self):
        return self.email
